    /*///'
 ./%'
,/'    .adAHHHAbn.           e b  i   o    s      |     |    |   |  | ||
%/    adAHY"'"YHAbn
//  .dHY"'     '"YHb.
//  dHY'         'YHb                       ___     ___
//  HH1           1HH                      /  /    /__/     /
//  HH1           1HH  .d888b.            /__/pen /repress /nterface
//  9Hb.         .dHP  887"88b
//  'VHAn.     .aAHA'   "  ;88
//    9HHAbo.adAHAP       4388b        d8888b .d88b 88b 8   8 .d88b 88b
//     '"YTHHHTY"'          781        8b.    8' .7 8 8 8   8 8' .7 8 8
//                     .   ;d81        '888b. 8 .7  8   9. .7 8 .7  8
//                    u8&oo78P'           'V8 8.7   8   '9.7' 8.7   8
//                     '98887'        o888888 '5880 8    'V'  '5880 8
//
//  O3  --  ebios opi-server  ---------------------------- ---- --- -- -
//  Copyright (c) ebios Informationssysteme
//  Written by Marcus Warga
/
                            //////////////////////////////////////////*/
#include <string>
#include <fstream>
#include <iostream>
#include <new>
using namespace std;
#include "o3instream.h"
#include "o3text.h"
#include "o3comment.h"
#include "o3bin.h"
#include "o3exceptions.h"
#include "o3literals.h"

InputStream *inputstream;

  //////////////////////////////////////////////////////////////////////////////////////////////////
 //..public......................................................................................//
//////////////////////////////////////////////////////////////////////////////////////////////////

InputStream::InputStream() : linebuffer(65536)
{
	binarydatafollows = false;
	endofLOb = true;
}

InputStream::~InputStream()
{
	if (!usecin)
		ifstream::close();
}

void InputStream::open(const string& filename)
{
	InputStream::filename = filename;
	ifstream::open(filename.c_str(), ios::in | ios::binary);

	if (fail())
		throw ExceptionFOpenFailed(filename, __FILE__, __LINE__);

	usecin = false;
}

void InputStream::usecinstream()
{
	usecin = true;
}

LOb* InputStream::read()  // the LOb-producer
{
	if (!binarydatafollows)	{
		const string& tmp = linebuffer.getLine();
//		cout << tmp << "\n";

		if (tmp[0] == '%') { // comment?
			CommentLOb *comment;
			try {
				comment = new CommentLOb();
			}
			catch(bad_alloc) {
				throw ExceptionOutOfMemory(__FILE__, __LINE__);
			}

			comment->assign(tmp);
			if (CMP(comment->getParm(0), Literals::PostScript::BeginData))
				onBeginData(*comment);
			else if (CMP(comment->getParm(0), Literals::PostScript::BeginBinary))
				onBeginBinary(*comment);

			return comment;
		} else {
			TextLOb *text;
			try {
				text = new TextLOb;
			}
			catch(bad_alloc) {
				throw ExceptionOutOfMemory(__FILE__, __LINE__);
			}

			text->assign(tmp);
			return text;
		}
	} else {
		if (linebuffer.isValid()) {
			BinLOb *bin = linebuffer.getBin(remainingdata); // get data if something is in the
			remainingdata -= bin->size();					// getline-buffer

			if (remainingdata == 0) {
				binarydatafollows = false;
				endofLOb = true;
			}

			if (streameof())
				endofLOb = true;

			return bin;
		}		
		// there was nothing in the getline-buffer, read from file..

		BinLOb *bin;
		try {
			bin = new BinLOb(1024*1024);
		}
		catch(bad_alloc) {
			throw ExceptionOutOfMemory(__FILE__, __LINE__);
		}

		if (remainingdata > bin->bufSize()) {

			if (usecin)
				cin.read(bin->rwData(), bin->bufSize());
			else
				ifstream::read(bin->rwData(), bin->bufSize());

			bin->size(bin->bufSize());
			remainingdata -= bin->bufSize();
		} else {

			if (usecin)
				cin.read(bin->rwData(), remainingdata);
			else
				ifstream::read(bin->rwData(), remainingdata);

			bin->size(remainingdata);
			remainingdata = 0;

			binarydatafollows = false;
			endofLOb = true;
		}

		if (streameof())
			endofLOb = true;

		return bin;
	}
}

bool InputStream::eof() const
{
	if (usecin) {
		if (cin.eof())
			return !linebuffer.isValid();
	} else {
		if (ifstream::eof())
			return !linebuffer.isValid();
	}

	return false;
}

bool InputStream::eoLOb() const
{
	return endofLOb;
}

bool InputStream::streameof() const
{
	if (usecin)
		return cin.eof();
	else
		return ifstream::eof();
}

size_t InputStream::gcount()
{
	if (usecin)
		return cin.gcount();
	else
		return ifstream::gcount();
}

const string& InputStream::src() const
{
	return (filename);
}

void InputStream::readRaw(char* dst, int bytes)
{
	if (usecin)
		cin.read(dst, bytes);
	else
		ifstream::read(dst, bytes);
}

  //////////////////////////////////////////////////////////////////////////////////////////////////
 //..private.....................................................................................//
//////////////////////////////////////////////////////////////////////////////////////////////////

bool InputStream::CMP(const string& str1, const char *str2)
{
#ifdef _MSC_VER

	if (!stricmp(str1.c_str(), str2))
		return true;
	else 
		return false;

#else

	if (!strcasecmp(str1.c_str(), str2))
		return true;
	else
		return false;

#endif
}

bool InputStream::CMP(const string& str1, const string& str2)
{
#ifdef _MSC_VER

	if (!stricmp(str1.c_str(), str2.c_str()))
		return true;
	else
		return false;

#else

	if (!strcasecmp(str1.c_str(), str2.c_str()))
		return true;
	else
		return false;

#endif
}

void InputStream::onBeginData(const CommentLOb& comment)
{
	remainingdata = atol(comment.getParm(1).c_str());
	endofLOb = false;
	binarydatafollows = true;
}

void InputStream::onBeginBinary(const CommentLOb& comment)
{
	remainingdata = atol(comment.getParm(1).c_str());
	endofLOb = false;
	binarydatafollows = true;
}
